@@ -19,7 +19,7 @@ import slack.errors as client_err
 from slack.web.client import WebClient
 
 
-class RTMClient(object):
+class RTMClient:
     """An RTMClient allows apps to communicate with the Slack Platform's RTM API.
 
     The event-driven architecture of this client allows you to simply
@@ -193,7 +193,7 @@ class RTMClient(object):
         Raises:
             SlackApiError: Unable to retrieve RTM URL from Slack.
         """
-        # TODO: Add Windows support for graceful shutdowns.
+        # NOTE: We may need to add Windows support for graceful shutdowns
         if os.name != "nt" and current_thread() == main_thread():
             signals = (signal.SIGHUP, signal.SIGTERM, signal.SIGINT)
             for s in signals:
@@ -371,7 +371,7 @@ class RTMClient(object):
             except (
                 client_err.SlackClientNotConnectedError,
                 client_err.SlackApiError,
-                # TODO: Catch websocket exceptions thrown by aiohttp.
+                # NOTE: We may want to catch WebSocket exceptions thrown by aiohttp too
             ) as exception:
                 await self._dispatch_event(event="error", data=exception)
                 error_code = (
@@ -394,18 +394,17 @@ class RTMClient(object):
 
     async def _read_messages(self):
         """Process messages received on the WebSocket connection."""
-        text_message_callback_executions: List[Future] = []
         while not self._stopped and self._websocket is not None:
-            for future in text_message_callback_executions:
-                if future.done():
-                    text_message_callback_executions.remove(future)
-
             try:
                 # Wait for a message to be received, but timeout after a second so that
                 # we can check if the socket has been closed, or if self._stopped is
                 # True
                 message = await self._websocket.receive(timeout=1)
             except asyncio.TimeoutError:
+                if self._websocket is None:
+                    # For some reason, the connection unexpectedly doesn't exist here.
+                    # In the case, this method can do nothing.
+                    return
                 if not self._websocket.closed:
                     # We didn't receive a message within the timeout interval, but
                     # aiohttp hasn't closed the socket, so ping responses must still be
@@ -419,34 +418,22 @@ class RTMClient(object):
                 )
                 self._websocket = None
                 await self._dispatch_event(event="close")
-                num_of_running_callbacks = len(text_message_callback_executions)
-                if num_of_running_callbacks > 0:
-                    self._logger.info(
-                        "WebSocket connection has been closed "
-                        f"though {num_of_running_callbacks} callback executions were still in progress"
-                    )
                 return
 
             if message.type == aiohttp.WSMsgType.TEXT:
-                payload = message.json()
-                event = payload.pop("type", "Unknown")
-
-                async def run_dispatch_event():
-                    try:
-                        await self._dispatch_event(event, data=payload)
-                    except Exception as err:
-                        data = message.data if message else message
-                        self._logger.info(
-                            f"Caught a raised exception ({err}) while dispatching a TEXT message ({data})"
-                        )
-                        # Raised exceptions here happen in users' code and were just unhandled.
-                        # As they're not intended for closing current WebSocket connection,
-                        # this exception should not be propagated to higher level (#_connect_and_read()).
-                        return
-
-                # Asynchronously run callbacks to handle simultaneous incoming messages from Slack
-                f = asyncio.ensure_future(run_dispatch_event())
-                text_message_callback_executions.append(f)
+                try:
+                    payload = message.json()
+                    event = payload.pop("type", "Unknown")
+                    await self._dispatch_event(event, data=payload)
+                except Exception as err:  # skipcq: PYL-W0703
+                    data = message.data if message else message
+                    self._logger.info(
+                        f"Caught a raised exception ({err}) while dispatching a TEXT message ({data})"
+                    )
+                    # Raised exceptions here happen in users' code and were just unhandled.
+                    # As they're not intended for closing current WebSocket connection,
+                    # this exception should not be propagated to higher level (#_connect_and_read()).
+                    continue
             elif message.type == aiohttp.WSMsgType.ERROR:
                 self._logger.error("Received an error on the websocket: %r", message)
                 await self._dispatch_event(event="error", data=message)
@@ -479,6 +466,8 @@ class RTMClient(object):
                 }
             }
         """
+        if self._logger.level <= logging.DEBUG:
+            self._logger.debug("Received an event: '%s' - %s", event, data)
         for callback in self._callbacks[event]:
             self._logger.debug(
                 "Running %s callbacks for event: '%s'",
@@ -588,7 +577,9 @@ class RTMClient(object):
         futures = []
         close_method = getattr(self._websocket, "close", None)
         if callable(close_method):
-            future = asyncio.ensure_future(close_method(), loop=self._event_loop)
+            future = asyncio.ensure_future(
+                close_method(), loop=self._event_loop,  # skipcq: PYL-E1102
+            )
             futures.append(future)
         self._websocket = None
         event_f = asyncio.ensure_future(
